@@ -1049,10 +1049,25 @@ pub enum InstructionErrorFieldless {
 }
 
 #[derive(FromPyObject, Clone, PartialEq, Eq, Serialize, Deserialize, Debug)]
+pub enum InstructionErrorTagged {
+    Custom(InstructionErrorCustom),
+    BorshIoError(InstructionErrorBorshIO),
+}
+
+impl IntoPy<PyObject> for InstructionErrorTagged {
+    fn into_py(self, py: Python<'_>) -> PyObject {
+        match self {
+            Self::Custom(x) => x.into_py(py),
+            Self::BorshIoError(x) => x.into_py(py),
+        }
+    }
+}
+
+#[derive(FromPyObject, Clone, PartialEq, Eq, Serialize, Deserialize, Debug)]
+#[serde(untagged)]
 pub enum InstructionErrorType {
     Fieldless(InstructionErrorFieldless),
-    Custom(InstructionErrorCustom),
-    BorshIO(InstructionErrorBorshIO),
+    Tagged(InstructionErrorTagged),
 }
 
 impl Default for InstructionErrorType {
@@ -1064,8 +1079,10 @@ impl Default for InstructionErrorType {
 impl From<InstructionErrorType> for InstructionErrorOriginal {
     fn from(w: InstructionErrorType) -> Self {
         match w {
-            InstructionErrorType::Custom(custom) => Self::Custom(custom.0),
-            InstructionErrorType::BorshIO(borsh_io) => Self::BorshIoError(borsh_io.0),
+            InstructionErrorType::Tagged(t) => match t {
+                InstructionErrorTagged::Custom(custom) => Self::Custom(custom.0),
+                InstructionErrorTagged::BorshIoError(borsh_io) => Self::BorshIoError(borsh_io.0),
+            },
             InstructionErrorType::Fieldless(f) => match f {
                 InstructionErrorFieldless::GenericError => Self::GenericError,
                 InstructionErrorFieldless::InvalidArgument => Self::InvalidArgument,
@@ -1145,10 +1162,12 @@ impl From<InstructionErrorType> for InstructionErrorOriginal {
 impl From<InstructionErrorOriginal> for InstructionErrorType {
     fn from(e: InstructionErrorOriginal) -> Self {
         match e {
-            InstructionErrorOriginal::Custom(code) => Self::Custom(InstructionErrorCustom(code)),
-            InstructionErrorOriginal::BorshIoError(val) => {
-                Self::BorshIO(InstructionErrorBorshIO(val))
+            InstructionErrorOriginal::Custom(code) => {
+                Self::Tagged(InstructionErrorTagged::Custom(InstructionErrorCustom(code)))
             }
+            InstructionErrorOriginal::BorshIoError(val) => Self::Tagged(
+                InstructionErrorTagged::BorshIoError(InstructionErrorBorshIO(val)),
+            ),
             InstructionErrorOriginal::GenericError => {
                 Self::Fieldless(InstructionErrorFieldless::GenericError)
             }
@@ -1307,8 +1326,7 @@ impl IntoPy<PyObject> for InstructionErrorType {
     fn into_py(self, py: Python<'_>) -> PyObject {
         match self {
             Self::Fieldless(f) => f.into_py(py),
-            Self::Custom(m) => m.into_py(py),
-            Self::BorshIO(b) => b.into_py(py),
+            Self::Tagged(t) => t.into_py(py),
         }
     }
 }
@@ -1411,13 +1429,29 @@ pub enum TransactionErrorFieldless {
 }
 
 #[derive(FromPyObject, Clone, PartialEq, Eq, Serialize, Deserialize, Debug)]
-pub enum TransactionErrorType {
-    Fieldless(TransactionErrorFieldless),
+pub enum TransactionErrorTypeTagged {
     InstructionError(TransactionErrorInstructionError),
     DuplicateInstruction(TransactionErrorDuplicateInstruction),
     InsufficientFundsForRent(TransactionErrorInsufficientFundsForRent),
 }
 
+impl IntoPy<PyObject> for TransactionErrorTypeTagged {
+    fn into_py(self, py: Python<'_>) -> PyObject {
+        match self {
+            Self::InstructionError(e) => e.into_py(py),
+            Self::DuplicateInstruction(d) => d.into_py(py),
+            Self::InsufficientFundsForRent(i) => i.into_py(py),
+        }
+    }
+}
+
+#[derive(FromPyObject, Clone, PartialEq, Eq, Serialize, Deserialize, Debug)]
+#[serde(untagged)]
+pub enum TransactionErrorType {
+    Fieldless(TransactionErrorFieldless),
+    Tagged(TransactionErrorTypeTagged),
+}
+
 impl Default for TransactionErrorType {
     fn default() -> Self {
         Self::Fieldless(TransactionErrorFieldless::AccountInUse)
@@ -1427,12 +1461,18 @@ impl Default for TransactionErrorType {
 impl From<TransactionErrorType> for TransactionErrorOriginal {
     fn from(w: TransactionErrorType) -> Self {
         match w {
-            TransactionErrorType::InstructionError(e) => {
-                Self::InstructionError(e.0 .0, e.0 .1.into())
-            }
-            TransactionErrorType::DuplicateInstruction(e) => Self::DuplicateInstruction(e.0),
-            TransactionErrorType::InsufficientFundsForRent(e) => Self::InsufficientFundsForRent {
-                account_index: e.account_index,
+            TransactionErrorType::Tagged(t) => match t {
+                TransactionErrorTypeTagged::InstructionError(e) => {
+                    Self::InstructionError(e.0 .0, e.0 .1.into())
+                }
+                TransactionErrorTypeTagged::DuplicateInstruction(e) => {
+                    Self::DuplicateInstruction(e.0)
+                }
+                TransactionErrorTypeTagged::InsufficientFundsForRent(e) => {
+                    Self::InsufficientFundsForRent {
+                        account_index: e.account_index,
+                    }
+                }
             },
             TransactionErrorType::Fieldless(f) => match f {
                 TransactionErrorFieldless::AccountInUse => Self::AccountInUse,
@@ -1497,15 +1537,19 @@ impl From<TransactionErrorOriginal> for TransactionErrorType {
     fn from(w: TransactionErrorOriginal) -> Self {
         match w {
             TransactionErrorOriginal::InstructionError(index, err) => {
-                Self::InstructionError(TransactionErrorInstructionError((index, err.into())))
+                Self::Tagged(TransactionErrorTypeTagged::InstructionError(
+                    TransactionErrorInstructionError((index, err.into())),
+                ))
             }
             TransactionErrorOriginal::DuplicateInstruction(index) => {
-                Self::DuplicateInstruction(TransactionErrorDuplicateInstruction(index))
+                Self::Tagged(TransactionErrorTypeTagged::DuplicateInstruction(
+                    TransactionErrorDuplicateInstruction(index),
+                ))
             }
             TransactionErrorOriginal::InsufficientFundsForRent { account_index } => {
-                Self::InsufficientFundsForRent(TransactionErrorInsufficientFundsForRent {
-                    account_index,
-                })
+                Self::Tagged(TransactionErrorTypeTagged::InsufficientFundsForRent(
+                    TransactionErrorInsufficientFundsForRent { account_index },
+                ))
             }
             TransactionErrorOriginal::AccountInUse => {
                 Self::Fieldless(TransactionErrorFieldless::AccountInUse)
@@ -1602,9 +1646,7 @@ impl IntoPy<PyObject> for TransactionErrorType {
     fn into_py(self, py: Python<'_>) -> PyObject {
         match self {
             Self::Fieldless(f) => f.into_py(py),
-            Self::InstructionError(e) => e.into_py(py),
-            Self::DuplicateInstruction(d) => d.into_py(py),
-            Self::InsufficientFundsForRent(i) => i.into_py(py),
+            Self::Tagged(t) => t.into_py(py),
         }
     }
 }
